@@ -61,4 +61,6 @@
 #include "fx_pggroup.h"
 #include "fx_pgprofile.h"
 #include "fx_smsstat.h"
+#include "fx_idle.h"
+#include "fx_conn.h"
 #endif
