@@ -25,7 +25,7 @@
 /*private method*/
 static char* generate_auth_body(User* user);
 static void parse_personal_info(xmlNodePtr node , User* user);
-static void parse_contact_list(xmlNodePtr node , User* user);
+static void parse_contact_list(xmlNodePtr node , User* user, int *group_count, int *buddy_count);
 static void parse_stranger_list(xmlNodePtr node , User* user);
 static void parse_ssi_auth_success(xmlNodePtr node , User* user);
 static void parse_ssi_auth_failed(xmlNodePtr node , User* user);
@@ -37,7 +37,6 @@ static char* hash_password_v4(const char* userid , const char* password);
 static char* generate_cnouce() ;
 static unsigned char* decode_base64(const char* in , int* len);
 
-
 char* generate_response(const char* nouce , const char* userid 
 		, const char* password , const char* publickey , const char* key)
 {
@@ -51,8 +50,6 @@ char* generate_response(const char* nouce , const char* userid
 	int nonce_len , aeskey_len , psd_len;
 	RSA *r = RSA_new();
 
-	DEBUG_FOOTPRINT();
-
 	key = NULL;
 
 	bzero(modulus , sizeof(modulus));
@@ -82,7 +79,8 @@ char* generate_response(const char* nouce , const char* userid
 	out =  (unsigned char*)malloc(flen);
 	memset(out , 0 , flen);
 	debug_info("Start encrypting response");
-	ret = RSA_public_encrypt(nonce_len + aeskey_len + psd_len, res , out, r, RSA_PKCS1_PADDING);
+	ret = RSA_public_encrypt(nonce_len + aeskey_len + psd_len,
+			res , out, r, RSA_PKCS1_PADDING);
 	if (ret < 0){
 		debug_info("Encrypt response failed!");
 		return NULL;
@@ -111,8 +109,6 @@ void generate_pic_code(User* user)
 	xmlDocPtr doc;
 	xmlNodePtr node;
 
-	DEBUG_FOOTPRINT();
-
 	memset(buf , 0 , sizeof(buf));
 	ip = get_ip_by_name(NAVIGATION_URI);
 	FetionConnection* con = tcp_connection_new();
@@ -165,8 +161,6 @@ char* ssi_auth_action(User* user)
 	int passwordType;
 	int ret;
 	
-	DEBUG_FOOTPRINT();
-
 	debug_info("Initialize ssi authentication action");
 	password = hash_password_v4(user->userId , user->password);
 	bzero(noUri , sizeof(noUri));
@@ -223,7 +217,6 @@ char* sipc_reg_action(User* user)
 
 	FetionSip* sip = user->sip;
 
-	DEBUG_FOOTPRINT();
 	debug_info("Initialize sipc registeration action");
 
 	fetion_sip_set_type(sip , SIP_REGISTER);
@@ -238,9 +231,9 @@ char* sipc_reg_action(User* user)
 	tcp_connection_send(sip->tcp , sipmsg , strlen(sipmsg));
 	free(sipmsg);
 	sipmsg = (char*)malloc(1024);
-	bzero(sipmsg , 1024);
-	if(tcp_connection_recv(sip->tcp , sipmsg , 1024) <= 0){
-		debug_info("Net work error occured here");
+	memset(sipmsg, 0 , 1024);
+	if(tcp_connection_recv(sip->tcp , sipmsg , 1023) <= 0){
+		debug_info("Network error occured here");
 		return NULL;
 	}
 	return sipmsg;
@@ -255,7 +248,6 @@ char* sipc_aut_action(User* user , const char* response)
 	SipHeader* ackheader = NULL;
 	FetionSip* sip = user->sip;
 
-	DEBUG_FOOTPRINT();
 	debug_info("Initialize sipc authencation action");
 
 	xml = generate_auth_body(user);
@@ -279,6 +271,7 @@ char* sipc_aut_action(User* user , const char* response)
 
 	tcp_connection_send(sip->tcp , sipmsg , strlen(sipmsg));
 	res = fetion_sip_get_response(sip);
+	debug_info("Got sipc response");
 	free(sipmsg);
 	return res;
 }
@@ -291,7 +284,6 @@ void parse_ssi_auth_response(const char* ssi_response , User* user)
 	char* xml = strstr(ssi_response , "\r\n\r\n") + 4;
 	int n;
 
-	DEBUG_FOOTPRINT();
 	if(strstr(ssi_response , "ssic=")){
 		pos = strstr(ssi_response , "ssic=") + 5;
 		n = strlen(pos) - strlen(strstr(pos , ";"));
@@ -324,8 +316,6 @@ void parse_sipc_reg_response(const char* reg_response , char** nouce , char** ke
 	char* pos;
 	int n;
 
-	DEBUG_FOOTPRINT();
-
 	fetion_sip_get_attr(reg_response , "W" , digest);
 
 	pos = strstr(digest , "nonce") + 7;
@@ -369,7 +359,7 @@ static void parse_sms_frequency(xmlNodePtr node , User *user)
 	}
 }
 
-int parse_sipc_auth_response(const char* auth_response , User* user)
+int parse_sipc_auth_response(const char* auth_response , User* user, int *group_count, int *buddy_count)
 {
 	char *pos;
 	xmlChar* buf = NULL;
@@ -379,8 +369,6 @@ int parse_sipc_auth_response(const char* auth_response , User* user)
 	xmlNodePtr node1 = NULL;
 	int code;
 
-	DEBUG_FOOTPRINT();
-
 	code = fetion_sip_get_code(auth_response);
 	user->loginStatus = code;
 
@@ -395,8 +383,15 @@ int parse_sipc_auth_response(const char* auth_response , User* user)
 		debug_error("Sipc authentication failed");
 		return -1;
 	}
+	if(!strstr(auth_response, "\r\n\r\n"))
+		return -1;
+
 	pos = strstr(auth_response , "\r\n\r\n") + 4;
-	doc = xmlReadMemory( pos , strlen(pos) , NULL , "UTF-8" , XML_PARSE_RECOVER);
+	if(!pos)
+		return -1;
+	doc = xmlParseMemory(pos , strlen(pos));
+	if(!doc)
+		return -1;
 	rootnode = xmlDocGetRootElement(doc); 
 	node = rootnode->xmlChildrenNode;
 	buf = xmlGetProp(node , BAD_CAST "public-ip");
@@ -416,20 +411,22 @@ int parse_sipc_auth_response(const char* auth_response , User* user)
 	strcpy(user->customConfigVersion , (char*)buf);
 	xmlFree(buf);
 	buf = xmlNodeGetContent(node1);
-	user->customConfig = malloc(strlen((char*)buf) + 1);
-	memset(user->customConfig , 0 , strlen((char*)buf) + 1);
-	strcpy(user->customConfig , (char*)buf);
+	if(xmlStrlen(buf) > 0){
+		user->customConfig = malloc(strlen((char*)buf) + 1);
+		memset(user->customConfig , 0 , strlen((char*)buf) + 1);
+		strcpy(user->customConfig , (char*)buf);
+	}
 	xmlFree(buf);
 	node1 = xml_goto_node(node , "contact-list");
-	parse_contact_list(node1 , user);
+	parse_contact_list(node1 , user, group_count, buddy_count);
 	node1 = xml_goto_node(node , "chat-friends");
-	if(node1 != NULL){
+	if(node1)
 		parse_stranger_list(node1 , user);
-	}
+	
 	node1 = xml_goto_node(node , "quota-frequency");
-	if(node1 != NULL){
+	if(node1)
 		parse_sms_frequency(node1 , user);
-	}
+	
 	xmlFreeDoc(doc);
 	return 1;
 }
@@ -460,8 +457,6 @@ static char* generate_auth_body(User* user)
 	xmlNodePtr node = NULL;
 	xmlNodePtr node1 = NULL;
 
-	DEBUG_FOOTPRINT();
-
 	doc = xmlParseMemory( basexml , strlen(basexml));
 	rootnode = xmlDocGetRootElement(doc); 
 	node = xmlNewChild(rootnode , NULL , BAD_CAST "device" , NULL);
@@ -474,18 +469,17 @@ static char* generate_auth_body(User* user)
 	xmlNewProp(node , BAD_CAST "mobile-no" , BAD_CAST user->mobileno);
 	xmlNewProp(node , BAD_CAST "user-id" , BAD_CAST user->userId);
 	node1 = xmlNewChild(node , NULL , BAD_CAST "personal" , NULL);
-	xmlNewProp(node1 , BAD_CAST "version" , BAD_CAST "0");//user->personalVersion);
+	xmlNewProp(node1 , BAD_CAST "version" , BAD_CAST user->personalVersion);
 	xmlNewProp(node1 , BAD_CAST "attributes" , BAD_CAST "v4default");
 	node1 = xmlNewChild(node , NULL , BAD_CAST "custom-config" , NULL);
-	xmlNewProp(node1 , BAD_CAST "version" , BAD_CAST "0");// user->customConfigVersion);
+	xmlNewProp(node1 , BAD_CAST "version" , BAD_CAST user->customConfigVersion);
 	node1 = xmlNewChild(node , NULL , BAD_CAST "contact-list" , NULL);
-	xmlNewProp(node1 , BAD_CAST "version" , BAD_CAST "0");//user->contactVersion);
+	xmlNewProp(node1 , BAD_CAST "version" , BAD_CAST user->contactVersion);
 	xmlNewProp(node1 , BAD_CAST "buddy-attributes" , BAD_CAST "v4default");
 	node = xmlNewChild(rootnode , NULL , BAD_CAST "credentials" , NULL);
 	xmlNewProp(node , BAD_CAST "domains" , BAD_CAST "fetion.com.cn");
 	node = xmlNewChild(rootnode , NULL , BAD_CAST "presence" , NULL);
 	node1 = xmlNewChild(node , NULL , BAD_CAST "basic" , NULL);
-	bzero(state , sizeof(state));
 	sprintf(state , "%d" , user->state);
 	xmlNewProp(node1 , BAD_CAST "value" , BAD_CAST state);
 	xmlNewProp(node1 , BAD_CAST "desc" , BAD_CAST "");
@@ -499,8 +493,6 @@ static void parse_personal_info(xmlNodePtr node , User* user)
 	char *pos;
 	int n;
 	
-	DEBUG_FOOTPRINT();
-
 	buf = xmlGetProp(node , BAD_CAST "version");
 	strcpy(user->personalVersion , (char*)buf);
 	xmlFree(buf);
@@ -566,27 +558,34 @@ static void parse_personal_info(xmlNodePtr node , User* user)
 		xmlFree(buf);
 	}
 }
-static void parse_contact_list(xmlNodePtr node , User* user)
+static void parse_contact_list(xmlNodePtr node, User* user,
+				int *group_count, int *buddy_count)
 {
 	xmlChar* buf = NULL;
 	xmlNodePtr node1 , node2;
 	Group* group = NULL;
 	Contact* contact = NULL;
 	int hasGroup = 1 , hasBuddy = 1;
+	int nr = 0;
 
-	DEBUG_FOOTPRINT();
+	*group_count = 0;
+	*buddy_count = 0;
 
-//	buf = xmlGetProp(node , BAD_CAST "version");
+	buf = xmlGetProp(node , BAD_CAST "version");
 	debug_info("Start reading contact list ");
-//	if(strcmp(user->contactVersion , (char*) buf) == 0){
-//		debug_info("Contact list is the same as that stored in the local disk!");
-//		return ;
-//	}
-//	strcpy(user->contactVersion , (char*)buf);
-//	xmlFree(buf);
+	if(strcmp(user->contactVersion , (char*) buf) == 0){
+		debug_info("Contact list is the same as that stored in the local disk!");
+		return ;
+	}
+	strcpy(user->contactVersion , (char*)buf);
+	xmlFree(buf);
 	node1 = xml_goto_node(node , "buddy-lists");
 	node2 = node1->xmlChildrenNode;
+	user->groupCount = 0;
+
 	while(node2 != NULL){
+		hasGroup = 1;
+
 		buf = xmlGetProp(node2 , BAD_CAST "id");
 		group = fetion_group_list_find_by_id(user->groupList , atoi((char*)buf));
 		if(group == NULL){
@@ -598,6 +597,10 @@ static void parse_contact_list(xmlNodePtr node , User* user)
 		buf = xmlGetProp(node2 , BAD_CAST "name");
 		strcpy(group->groupname , (char*)buf);
 		xmlFree(buf);
+
+		nr ++;
+		group->dirty = 1;
+		user->groupCount ++;
 		
 		if(hasGroup == 0){
 			fetion_group_list_append(user->groupList , group);
@@ -605,9 +608,17 @@ static void parse_contact_list(xmlNodePtr node , User* user)
 		}
 		node2 = node2->next;
 	}
+
+	*group_count = nr;
+	nr = 0;
+
 	node1 = xml_goto_node(node , "buddies");
 	node1 = node1->xmlChildrenNode;
+	user->contactCount = 0;
+
 	while(node1 != NULL){
+		hasBuddy = 1;
+
 		if(! xmlHasProp(node1 , BAD_CAST "i")){
 			node1 = node1->next;
 			continue;
@@ -620,6 +631,18 @@ static void parse_contact_list(xmlNodePtr node , User* user)
 		}
 		strcpy(contact->userId , (char*)buf);
 		xmlFree(buf);
+
+		/* maybe a buddy belongs to two groups */
+		if(contact->dirty == 1){
+			node = node->next;
+			continue;
+		}
+
+		user->contactCount ++;
+		/* set the dirty flags */
+		contact->dirty = 1;
+		nr ++;
+
 		if(xmlHasProp(node1 , BAD_CAST "n")){
 			buf = xmlGetProp(node1 , BAD_CAST "n");
 			strcpy(contact->localname , (char*)buf);
@@ -655,7 +678,7 @@ static void parse_contact_list(xmlNodePtr node , User* user)
 			xmlFree(buf);
 		}
 
-		strcpy(contact->portraitCrc , "unlogin");
+		strcpy(contact->portraitCrc , "0");
 
 		if(hasBuddy == 0){
 			fetion_contact_list_append(user->contactList , contact);
@@ -663,30 +686,38 @@ static void parse_contact_list(xmlNodePtr node , User* user)
 		}
 		node1 = node1->next;
 	}
+
+	*buddy_count = nr;
 	debug_info("Read contact list complete");
 }
+
+
 static void parse_stranger_list(xmlNodePtr node , User* user)
 {
 	xmlNodePtr node1 = node->xmlChildrenNode;
 	xmlChar *buf = NULL;
 	Contact *contact = NULL;
-	int hasBuddy = 1;
+	int hasBuddy;
 	while(node1 != NULL)
 	{
-		buf = xmlGetProp(node1 , BAD_CAST "u");
+		hasBuddy = 1;
+		user->contactCount ++;
+		buf = xmlGetProp(node1 , BAD_CAST "i");
 		contact = fetion_contact_list_find_by_userid(user->contactList , (char*)buf);
 		if(contact == NULL){
 			hasBuddy = 0;
 			contact = fetion_contact_new();
 		}
-		strcpy(contact->sipuri , (char*)buf);
-		xmlFree(buf);
-		buf = xmlGetProp(node1 , BAD_CAST "i");
 		strcpy(contact->userId , (char*)buf);
+		xmlFree(buf);
+		buf = xmlGetProp(node1 , BAD_CAST "u");
+		strcpy(contact->sipuri , (char*)buf);
 		contact->groupid = BUDDY_LIST_STRANGER;
-		if(hasBuddy == 0){
+		contact->dirty = 1;
+
+		if(hasBuddy == 0)
 			fetion_contact_list_append(user->contactList , contact);
-		}
+		
 		node1 = node1->next;
 	}
 }
