@@ -36,22 +36,14 @@ User* fetion_user_new(const char* no , const char* password)
 {
 	User* user = (User*)malloc(sizeof(User));
 	memset(user , 0 , sizeof(User));
-	if(strlen(no) == 11)
-	{
+	if(strlen(no) == 11){
 		strcpy(user->mobileno , no);
 		user->loginType = LOGIN_TYPE_MOBILENO;
-	}
-	else
-	{
+	}else{
 		strcpy(user->sId , no);
 		user->loginType = LOGIN_TYPE_FETIONNO;
 	}
 	strcpy(user->password , password);
-	strcpy(user->personalVersion , "0");
-	strcpy(user->contactVersion , "0");
-	strcpy(user->configServersVersion , "0");
-	strcpy(user->configHintsVersion , "0");
-	strcpy(user->customConfigVersion , "0");
 	user->contactList = fetion_contact_new();
 	user->groupList = fetion_group_new();
 	user->pggroup = NULL;
@@ -226,6 +218,12 @@ void fetion_group_list_prepend(Group* head , Group* group)
 	head->pre = group;
 }
 
+void fetion_group_list_remove(Group *group)
+{
+	group->next->pre = group->pre;
+	group->pre->next = group->next;
+}
+
 void fetion_group_remove(Group* head , int groupid)
 {
 	Group *gl_cur;
@@ -298,7 +296,6 @@ int fetion_user_upload_portrait(User* user , const char* filename)
 	filelength = ftell(f);
 	rewind(f);
 	debug_info("uploading portrait....");
-	bzero(http , sizeof(http));
 	sprintf(http , "POST /%s/setportrait.aspx HTTP/1.1\r\n"
 		    	   "Cookie: ssic=%s\r\n"
 				   "Accept: */*\r\n"
@@ -395,6 +392,8 @@ int fetion_user_download_portrait_with_uri(User *user , const char *sipuri
 	Config *config = user->config;
 	FetionConnection* tcp = NULL;
 	int i = 0 , isFirstChunk = 0 , chunkLength = 0 , imageLength = 0 , receivedLength = 0;
+	int ret;
+
 	ip = get_ip_by_name(server);
 	if(ip == NULL)
 	{
@@ -407,7 +406,6 @@ int fetion_user_download_portrait_with_uri(User *user , const char *sipuri
 	if(friendSid == NULL)
 		return -1;
 /*	open a file to write ,if not exist then create one*/
-	bzero(filename , sizeof(filename));
 	sprintf(filename , "%s/%s.jpg" , config->iconPath ,  friendSid);
 	free(friendSid);
 	encodedSipuri = http_connection_encode_url(sipuri);
@@ -422,17 +420,25 @@ int fetion_user_download_portrait_with_uri(User *user , const char *sipuri
 
 	tcp = tcp_connection_new();
 	if(config->proxy != NULL && config->proxy->proxyEnabled)
-		tcp_connection_connect_with_proxy(tcp , ip , 80 , config->proxy);
+		ret = tcp_connection_connect_with_proxy(tcp , ip , 80 , config->proxy);
 	else
-		tcp_connection_connect(tcp , ip , 80);
+		ret = tcp_connection_connect(tcp , ip , 80);
+	if(ret < 0){
+		debug_error("connect to portrait server:%s",
+				strerror(errno));
+		return -1;
+	}
 	free(ip);
-	tcp_connection_send(tcp , buf , strlen(buf));	
+	ret = tcp_connection_send(tcp , buf , strlen(buf));	
+	if(ret < 0)
+		return -1;
+
 	//read reply
 
 	/* 200 OK begin to download protrait ,
 	 * 302 need to redirect ,404 not found */
 	for(;;){
-		bzero(buf , sizeof(buf));
+		memset(buf, 0, sizeof(buf));
 		chunkLength = tcp_connection_recv(tcp , buf , sizeof(buf) -1);
 		if(chunkLength < 0)
 			break;
@@ -496,13 +502,15 @@ int fetion_user_download_portrait_with_uri(User *user , const char *sipuri
 	}
 redirect:
 	if(strcmp(replyCode , "302") == 0)
-		fetion_user_download_portrait_again(filename , buf , config->proxy);
+		ret = fetion_user_download_portrait_again(filename , buf , config->proxy);
 end:
 	if(f != NULL)
 		fclose(f);
 	tcp_connection_free(tcp);
 	tcp = NULL;
-	return 0;
+	if(ret < 0)
+		return -1;
+	return 1;
 }
 
 static int fetion_user_download_portrait_again(const char* filepath , const char* buf , Proxy* proxy)
@@ -518,6 +526,7 @@ static int fetion_user_download_portrait_again(const char* filepath , const char
 	char* pos = strstr(buf , "Location: ") ;
 	int chunkLength = 0 , imageLength = 0 , receivedLength = 0;
 	int i , n = 0;
+	int ret;
 	
 	int isFirstChunk = 0;
 	unsigned char img[2048] = { 0 };
@@ -534,7 +543,8 @@ static int fetion_user_download_portrait_again(const char* filepath , const char
 	strcpy(httpPath , pos);
 	sprintf(http , "GET %s HTTP/1.1\r\n"
 				   "User-Agent: IIC2.0/PC 3.3.0370\r\n"
-			 	   "Accept: image/pjpeg;image/jpeg;image/bmp;image/x-windows-bmp;image/png;image/gif\r\n"
+			 	   "Accept: image/pjpeg;image/jpeg;image/bmp;"
+				   "image/x-windows-bmp;image/png;image/gif\r\n"
 				   "Cache-Control: private\r\n"
 				   "Host: %s\r\n"
 				   "Connection: Keep-Alive\r\n\r\n" , httpPath , httpHost);
@@ -546,16 +556,21 @@ static int fetion_user_download_portrait_again(const char* filepath , const char
 	tcp = tcp_connection_new();
 
 	if(proxy != NULL && proxy->proxyEnabled)
-		tcp_connection_connect_with_proxy(tcp , ip , 80 , proxy);
+		ret = tcp_connection_connect_with_proxy(tcp , ip , 80 , proxy);
 	else
-		tcp_connection_connect(tcp , ip , 80);
+		ret = tcp_connection_connect(tcp , ip , 80);
+
+	if(ret < 0)
+		return -1;
 
 	free(ip);
-	tcp_connection_send(tcp , http , strlen(http));
+	ret = tcp_connection_send(tcp , http , strlen(http));
+	if(ret < 0)
+		return -1;
 	//read portrait data
 	f = fopen(filepath , "wb+");
 	for(;;){
-		bzero(img , sizeof(img));
+		memset(img, 0 , sizeof(img));
 		chunkLength = tcp_connection_recv(tcp , img , sizeof(img)-1);
 		if(chunkLength <= 0)
 			break;
@@ -660,9 +675,17 @@ Contact* fetion_user_parse_presence_body(const char* body , User* user)
 		if(xmlHasProp(cnode , BAD_CAST "p"))
 		{
 			pos = xmlGetProp(cnode , BAD_CAST "p");
+			if(strcmp(currentContact->portraitCrc, (char*)pos) == 0
+					|| strcmp((char*)pos, "0") == 0)
+				currentContact->imageChanged = 0;
+			else
+				currentContact->imageChanged = 1;
 			strcpy(currentContact->portraitCrc ,  (char*)pos);
 			xmlFree(pos);
+		}else{
+			currentContact->imageChanged = 0;
 		}
+
 		if(xmlHasProp(cnode , BAD_CAST "c"))
 		{
 			pos = xmlGetProp(cnode , BAD_CAST "c");
@@ -767,6 +790,115 @@ Contact* fetion_user_parse_syncuserinfo_body(const char* body , User* user)
 	xmlFreeDoc(doc);
 	return currentContact;
 }
+
+void fetion_user_save(User *user)
+{
+	char path[256];
+	char sql[4096];
+	char *errMsg;
+	sqlite3 *db;
+	Config *config = user->config;
+
+	sprintf(path, "%s/data.db",
+				   	config->userPath);
+
+	debug_info("Save user information");
+	if(sqlite3_open(path, &db)){
+		debug_error("open data.db:%s",
+					sqlite3_errmsg(db));
+		return;
+	}
+
+	sprintf(sql, "delete from user;");
+	if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+		sprintf(sql, "create table user ("
+					"sId,userId,mobileno,password,sipuri,"
+					"publicIp,lastLoginIp,lastLoginTime,"
+					"personalVersion,contactVersion,"
+					"nickname,impression,country,province,"
+					"city,gender,smsOnLineStatus,"
+					"customConfigVersion, customConfig,"
+					"boundToMobile);");
+		if(sqlite3_exec(db, sql, NULL, NULL, &errMsg)){
+			debug_error("create table user:%s",
+							errMsg ? errMsg : "");
+			sqlite3_close(db);
+			return;
+		}
+	}
+	snprintf(sql, sizeof(sql)-1, "insert into user "
+				"values ('%s','%s','%s','%s','%s',"
+				"'%s','%s','%s','%s','%s',"
+				"'%s','%s','%s','%s','%s',%d,'%s',"
+				" '%s', '%s',%d);",
+				user->sId, user->userId, user->mobileno,
+				user->password, user->sipuri, user->publicIp,
+				user->lastLoginIp, user->lastLoginTime,
+				user->personalVersion, user->contactVersion,
+				user->nickname, user->impression, user->country,
+				user->province, user->city, user->gender,
+				user->smsOnLineStatus, user->customConfigVersion,
+				user->customConfig, user->boundToMobile);
+	if(sqlite3_exec(db, sql, NULL, NULL, &errMsg))
+		debug_error("update user:%s\n%s",
+						errMsg ? errMsg : "", sql);
+
+	sqlite3_close(db);
+}
+
+void fetion_user_load(User *user)
+{
+	char path[256];
+	char sql[4096];
+	char *errMsg;
+	char **sqlres;
+	sqlite3 *db;
+	int ncols, nrows;
+	Config *config = user->config;
+
+	sprintf(path, "%s/data.db",
+				   	config->userPath);
+
+	debug_info("Load user information");
+	if(sqlite3_open(path, &db)){
+		debug_error("open data.db:%s",
+					sqlite3_errmsg(db));
+		return;
+	}
+
+	sprintf(sql, "select * from user;");
+	if(sqlite3_get_table(db, sql, &sqlres
+						, &nrows, &ncols, &errMsg)){
+		sqlite3_close(db);
+		return;
+	}
+
+	strcpy(user->sId , 				sqlres[ncols]);
+	strcpy(user->userId , 			sqlres[ncols+1]);
+	strcpy(user->mobileno , 		sqlres[ncols+2]);
+	strcpy(user->password , 		sqlres[ncols+3]);
+	strcpy(user->sipuri , 			sqlres[ncols+4]);
+	strcpy(user->publicIp , 		sqlres[ncols+5]);
+	strcpy(user->lastLoginIp , 		sqlres[ncols+6]);
+	strcpy(user->lastLoginTime , 	sqlres[ncols+7]);
+	strcpy(user->personalVersion , 	sqlres[ncols+8]);
+	strcpy(user->contactVersion , 	sqlres[ncols+9]);
+	strcpy(user->nickname , 		sqlres[ncols+10]);
+	strcpy(user->impression , 		sqlres[ncols+11]);
+	strcpy(user->country , 			sqlres[ncols+12]);
+	strcpy(user->province , 		sqlres[ncols+13]);
+	strcpy(user->city , 			sqlres[ncols+14]);
+	user->gender = 					atoi(sqlres[ncols+15]);
+	strcpy(user->smsOnLineStatus ,  sqlres[ncols+16]);
+	strcpy(user->customConfigVersion ,  sqlres[ncols+17]);
+	user->customConfig = (char*)malloc(strlen(sqlres[ncols+18])+1);
+	memset(user->customConfig, 0, strlen(sqlres[ncols+18])+1);
+	strcpy(user->customConfig, sqlres[ncols+18]);
+	user->boundToMobile = atoi(sqlres[ncols+19]);
+
+	sqlite3_close(db);
+}
+
 static char* generate_set_state_body(StateType state)	
 {
 	char s[5];
@@ -819,7 +951,6 @@ static char* generate_update_information_body(User* user)
 	cnode = xmlNewChild(node , NULL , BAD_CAST "personal" , NULL);
 	xmlNewProp(cnode , BAD_CAST "impresa" , BAD_CAST user->impression);
 	xmlNewProp(cnode , BAD_CAST "nickname" , BAD_CAST user->nickname);
-	bzero(gender , sizeof(gender));
 	sprintf(gender , "%d" , user->gender);
 	xmlNewProp(cnode , BAD_CAST "gender" , BAD_CAST gender);
 	xmlNewProp(cnode , BAD_CAST "version" , BAD_CAST "0");
@@ -888,7 +1019,6 @@ static char* generate_set_sms_status_body(int days)
 	node = xmlDocGetRootElement(doc);
 	node = xmlNewChild(node , NULL , BAD_CAST "userinfo" , NULL);
 	node = xmlNewChild(node , NULL , BAD_CAST "personal" , NULL);
-	memset(status , 0 , sizeof(status));
 	sprintf(status , "%d.00:00:00" , days);
 	xmlNewProp(node , BAD_CAST "sms-online-status" , BAD_CAST status);
 	xmlDocDumpMemory(doc , &res , NULL);
