@@ -30,8 +30,6 @@ static void fx_add_group_on_ok_clicked(GtkWidget* UNUSED(widget) , gpointer data
 	GtkTreeIter iter;
 	int groupid;
 	
-	DEBUG_FOOTPRINT();
-
 	groupid = fetion_buddylist_create(user , text);
 	if(groupid > 0 )
 	{
@@ -43,15 +41,12 @@ static void fx_add_group_on_ok_clicked(GtkWidget* UNUSED(widget) , gpointer data
 						 , G_ONLINE_COUNT_COL , 0
 						 , -1);
 		gtk_tree_view_set_model(tree , model);
-		fetion_buddylist_save(user);
 	}
 	gtk_dialog_response(GTK_DIALOG(fxaddgroup->dialog) , GTK_RESPONSE_CANCEL);
 }
 
 static void fx_add_group_on_cancel_clicked(GtkWidget* UNUSED(widget) , gpointer data)
 {
-	DEBUG_FOOTPRINT();
-
 	gtk_dialog_response(GTK_DIALOG(data) , GTK_RESPONSE_CANCEL);
 }
 
@@ -59,8 +54,6 @@ FxAddGroup* fx_add_group_new(FxMain* fxmain)
 {
 	FxAddGroup* fxaddgroup = (FxAddGroup*)malloc(sizeof(FxAddGroup));
 
-	DEBUG_FOOTPRINT();
-
 	memset(fxaddgroup , 0 , sizeof(FxAddGroup));
 	fxaddgroup->fxmain = fxmain;
 	return fxaddgroup;
@@ -70,8 +63,6 @@ void fx_add_group_initialize(FxAddGroup* fxaddgroup)
 {
 	GdkPixbuf* pb;
 
-	DEBUG_FOOTPRINT();
-
 	fxaddgroup->dialog = gtk_dialog_new();
 	pb = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"online.svg" , 22 , 22 , NULL);
 	gtk_dialog_set_has_separator(GTK_DIALOG(fxaddgroup->dialog) , FALSE);
@@ -103,7 +94,5 @@ void fx_add_group_initialize(FxAddGroup* fxaddgroup)
 
 void fx_add_group_free(FxAddGroup* fxaddgroup)
 {
-	DEBUG_FOOTPRINT();
-
 	free(fxaddgroup);
 }
