@@ -20,21 +20,18 @@
 
 #include "fx_include.h"
 
+extern int old_state;
 
 FxHead* fx_head_new()
 {
 	FxHead* fxhead = (FxHead*)malloc(sizeof(FxHead));
 
-	DEBUG_FOOTPRINT();
-
 	memset(fxhead , 0 , sizeof(FxHead));
 	return fxhead;
 }
 
 void fx_head_free(FxHead* fxhead)
 {
-	DEBUG_FOOTPRINT();
-
 	if(fxhead != NULL)
 		free(fxhead);
 }
@@ -50,9 +47,8 @@ void fx_head_initialize(FxMain* fxmain)
 	halign = gtk_alignment_new(0, 0 , 0 , 0);
 	gtk_container_add(GTK_CONTAINER(halign) , hbox);
 	
-	DEBUG_FOOTPRINT();
-
-	pb = gdk_pixbuf_new_from_file_at_scale(SKIN_DIR"fetion.svg" , 50 , 50 , TRUE , NULL);
+	pb = gdk_pixbuf_new_from_file_at_scale(SKIN_DIR"fetion.svg",
+			USER_PORTRAIT_SIZE, USER_PORTRAIT_SIZE, TRUE, NULL);
 	fxhead->portrait = gtk_image_new_from_pixbuf(pb);
 	g_object_unref(pb);
 
@@ -181,36 +177,30 @@ void fx_head_bind(FxMain* fxmain)
 	char* statename = NULL;
 	GdkPixbuf* portrait_pix = NULL;
 
-	DEBUG_FOOTPRINT();
-
-	bzero(name , sizeof(name));
 	bzero(fxhead->oldimpression , sizeof(fxhead->oldimpression));
 
 	sprintf(name , "<b>%s</b>"
-			, user->nickname == NULL ? user->sId : user->nickname );
+			, user->nickname == NULL ? user->sId : g_markup_escape_text(user->nickname, -1) );
 
-	gtk_label_set_markup(GTK_LABEL(fxhead->name_label) , name );
+	gtk_label_set_markup(GTK_LABEL(fxhead->name_label), name );
 	
 
 	strcpy(fxhead->oldimpression
 		, (strlen(user->impression) == 0 || user->impression == NULL)
 		? "Click here to input signature" : user->impression);
 
-	memset(tooltip , 0 , sizeof(tooltip));
-	sprintf(tooltip , "<b>%s</b>" , user->impression);
-	gtk_widget_set_tooltip_markup(fxhead->impre_label
-			, tooltip);
+	sprintf(tooltip, "<b>%s</b>", g_markup_escape_text(user->impression, strlen(user->impression)));
+	gtk_widget_set_tooltip_markup(fxhead->impre_label, tooltip);
 	escape_impression(fxhead->oldimpression);
-	gtk_label_set_text(GTK_LABEL(fxhead->impre_label) , fxhead->oldimpression);
-
-	bzero(name , sizeof(name));
+	gtk_label_set_text(GTK_LABEL(fxhead->impre_label), fxhead->oldimpression);
 
 	sprintf(name , "%s/%s.jpg" , config->iconPath , user->sId);
 
-	fetion_user_download_portrait(user , user->sipuri);
-	portrait_pix = gdk_pixbuf_new_from_file_at_size(name , 50 , 50 , NULL);
+	portrait_pix = gdk_pixbuf_new_from_file_at_size(name,
+			USER_PORTRAIT_SIZE, USER_PORTRAIT_SIZE, NULL);
 	if(! portrait_pix)
-		portrait_pix = gdk_pixbuf_new_from_file_at_size(name , 50 , 50 , NULL);
+		portrait_pix = gdk_pixbuf_new_from_file_at_size(name,
+				USER_PORTRAIT_SIZE, USER_PORTRAIT_SIZE, NULL);
 	if(portrait_pix)
 		gtk_image_set_from_pixbuf(GTK_IMAGE(fxhead->portrait) , portrait_pix);
 
@@ -226,8 +216,6 @@ void fx_head_set_state_image(FxMain* fxmain , StateType type)
 	GdkPixbuf *pixbuf;
 	char* statename = fx_util_get_state_name(type);
 
-	DEBUG_FOOTPRINT();
-
 	gtk_label_set_markup(GTK_LABEL(fxhead->state_label) , statename);
 	switch(type)
 	{
@@ -246,6 +234,12 @@ void fx_head_set_state_image(FxMain* fxmain , StateType type)
 			gtk_status_icon_set_from_file(fxmain->trayIcon
 										, SKIN_DIR"invisible.svg");
 			break;
+		case P_OFFLINE :
+			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"offline.svg" , 20 , 20 , NULL);
+			gtk_status_icon_set_from_file(fxmain->trayIcon
+										, SKIN_DIR"offline.svg");
+			gtk_widget_set_sensitive(fxhead->portrait, FALSE);
+			break;
 		default :
 			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"away.svg" , 20 , 20 , NULL);
 			gtk_status_icon_set_from_file(fxmain->trayIcon
@@ -267,8 +261,6 @@ void fx_head_popup_statemenu_func(GtkWidget* widget
 	GdkCursor *cursor;
 	GtkWidget *presence_menu = gtk_menu_new();
 
-	DEBUG_FOOTPRINT();
-
 	if(event->type == GDK_ENTER_NOTIFY) {
 		cursor = gdk_cursor_new (GDK_HAND2);
 		gdk_window_set_cursor(widget->window , cursor);
@@ -285,6 +277,7 @@ void fx_head_popup_statemenu_func(GtkWidget* widget
 	fx_head_create_presence_item(P_AWAY   , _("Leave") , presence_menu , fxmain);
 	fx_head_create_presence_item(P_BUSY   , _("Busy") , presence_menu , fxmain);
 	fx_head_create_presence_item(P_HIDDEN , _("Hide") , presence_menu , fxmain);
+	fx_head_create_presence_item(P_OFFLINE , _("Offline") , presence_menu , fxmain);
 
 	separator = gtk_separator_menu_item_new();
 	gtk_menu_shell_append(GTK_MENU_SHELL(presence_menu) , separator);
@@ -349,6 +342,9 @@ void fx_head_create_presence_item(int type , const char* message , GtkWidget* me
 		case P_HIDDEN :
 			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"invisible.svg" , 20 , 20 , NULL);
 			break;
+		case P_OFFLINE :
+			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"offline.svg", 20, 20, NULL);
+			break;
 		default :
 			pixbuf = gdk_pixbuf_new_from_file_at_size(SKIN_DIR"away.svg" , 20 , 20 , NULL);
 			break;
@@ -367,17 +363,17 @@ void fx_head_impre_event_func(GtkWidget* widget , GdkEventButton* event , gpoint
 	GdkCursor* cursor = NULL;
 	const char* text = NULL;
 
-	DEBUG_FOOTPRINT();
-
 	switch(event->type)
 	{
 		case GDK_BUTTON_PRESS :
-			gtk_widget_show(fxhead->impre_entry);
-			text = gtk_label_get_text(GTK_LABEL(fxhead->impre_label));
-			gtk_entry_set_text(GTK_ENTRY(fxhead->impre_entry) , text);
-			gtk_entry_select_region(GTK_ENTRY(fxhead->impre_entry) , 0 , strlen(text));
-			gtk_widget_grab_focus(fxhead->impre_entry);
-			gtk_widget_hide(widget);
+			if(fx_conn_check_action(fxmain)){
+				gtk_widget_show(fxhead->impre_entry);
+				text = gtk_label_get_text(GTK_LABEL(fxhead->impre_label));
+				gtk_entry_set_text(GTK_ENTRY(fxhead->impre_entry) , text);
+				gtk_entry_select_region(GTK_ENTRY(fxhead->impre_entry) , 0 , strlen(text));
+				gtk_widget_grab_focus(fxhead->impre_entry);
+				gtk_widget_hide(widget);
+			}
 			break;
 		case GDK_ENTER_NOTIFY :
 			gtk_widget_modify_bg(fxhead->impre_box, GTK_STATE_NORMAL, &color);
@@ -399,8 +395,6 @@ gboolean fx_head_impre_focus_out_func(GtkWidget* UNUSED(widget)
 	FxMain* fxmain = (FxMain*)data;
 	FxHead* fxhead = fxmain->headPanel;
 
-	DEBUG_FOOTPRINT();
-
 	gtk_widget_show(fxhead->impre_box);
 	gtk_widget_hide(fxhead->impre_entry);
 	return FALSE;
@@ -411,11 +405,9 @@ gboolean fx_head_impre_activate_func(GtkWidget* widget , gpointer data)
 	FxMain* fxmain = (FxMain*)data;
 	FxHead* fxhead = fxmain->headPanel;
 
-	char tooltip[1024];
-	const char* impression = gtk_entry_get_text(GTK_ENTRY(widget));
+	gchar tooltip[1024];
+	const gchar* impression = gtk_entry_get_text(GTK_ENTRY(widget));
 	
-	DEBUG_FOOTPRINT();
-
 	gtk_widget_show(fxhead->impre_box);
 	gtk_widget_hide(widget);
 	if(strcmp(impression , fxhead->oldimpression) == 0)
@@ -426,28 +418,61 @@ gboolean fx_head_impre_activate_func(GtkWidget* widget , gpointer data)
 		bzero(fxhead->oldimpression , sizeof(fxhead->oldimpression));
 		strcpy(fxhead->oldimpression , impression);
 		bzero(tooltip , sizeof(tooltip));
-		sprintf(tooltip , "<b>%s</b>" , impression);
+		sprintf(tooltip , "<b>%s</b>" , g_markup_escape_text(impression, -1));
 		gtk_widget_set_tooltip_markup(fxhead->impre_label , tooltip);
 	}
 	return TRUE;
 }
+
+static void* reconnection_func(void *data)
+{
+	typedef struct {
+		FxMain    *fxmain;
+		StateType  type;
+	} Args;
+	Args     *args = (Args*)data;
+	FxMain   *fxmain = args->fxmain;
+	FxHead   *fxhead = fxmain->headPanel;
+
+	gdk_threads_enter();
+	gtk_image_set_from_file(GTK_IMAGE(fxhead->portrait),
+			SKIN_DIR"reconnecting.gif");
+	gdk_threads_leave();
+
+	fx_conn_reconnect(fxmain, args->type);
+
+	return NULL;
+}
+
 void fx_head_change_state_func(GtkWidget* UNUSED(widget) , gpointer data)
 {
-	typedef struct 
-	{
-		FxMain* fxmain;
-		StateType type;
+	typedef struct {
+		FxMain    *fxmain;
+		StateType  type;
 	} Args;
-	Args *args = (Args*)data;
-	FxMain* fxmain = args->fxmain;
-	User* user = fxmain->user;
+	Args   *args = (Args*)data;
+	FxMain *fxmain = args->fxmain;
+	User   *user = fxmain->user;
 
-	DEBUG_FOOTPRINT();
-
-	if(fetion_user_set_state(user , args->type) > 0)
-	{
-		fx_head_set_state_image(fxmain , args->type);
+	if(args->type == P_OFFLINE){
+		fx_conn_offline(fxmain);
+		g_free(args);
+		return;
 	}
+
+	if(user->state == P_OFFLINE){
+		g_thread_create(reconnection_func,
+				args, FALSE, NULL);
+		return;
+	}
+
+	if(fetion_user_set_state(user , args->type) > 0){
+		fx_head_set_state_image(fxmain , args->type);
+		old_state = args->type;
+	}
+
+	g_free(args);
+
 }
 void fx_head_change_portrait_func(GtkWidget* widget , GdkEventButton* event , gpointer data)
 {
@@ -462,8 +487,6 @@ void fx_head_change_portrait_func(GtkWidget* widget , GdkEventButton* event , gp
 	} *args = (struct Args*)malloc(sizeof(struct Args));
 	int response;
 
-	DEBUG_FOOTPRINT();
-
 	if(event->type == GDK_ENTER_NOTIFY)
 	{
 		cursor = gdk_cursor_new (GDK_HAND2);
@@ -478,6 +501,9 @@ void fx_head_change_portrait_func(GtkWidget* widget , GdkEventButton* event , gp
 		return;
 	}
 
+	if(!fx_conn_check_action(fxmain))
+		return;
+
 	filechooser = gtk_file_chooser_dialog_new(_("Choose the avatar file to upload")
 							   , NULL , GTK_FILE_CHOOSER_ACTION_OPEN
 							   , _("Upload") , 1 , _("Cancel") , 2 , NULL);
@@ -505,31 +531,32 @@ void* fx_head_change_portrait_thread(void* data)
 		FxMain* fxmain;
 		char filename[1024];
 	} *args = (struct Args*)data;
-	FxMain* fxmain = args->fxmain;
-	FxHead* fxhead = fxmain->headPanel;
-	Config* config = fxmain->user->config;
-	char filepath[128];
-	GtkWidget *dialog = NULL;
-	GdkPixbuf* pb = NULL;
-	
-	DEBUG_FOOTPRINT();
+	FxMain     *fxmain;
+	FxHead     *fxhead;
+	Config     *config;
+	gchar       filepath[128];
+	GtkWidget  *dialog;
+	GdkPixbuf  *pb;
 
-	bzero(filepath , sizeof(filepath));
-	if(fetion_user_upload_portrait(fxmain->user , args->filename) > 0)
-	{
-		
-		gdk_threads_enter();
+	fxmain = args->fxmain;
+	fxhead = fxmain->headPanel;
+	config = fxmain->user->config;
+	
+	if(fetion_user_upload_portrait(fxmain->user , args->filename) > 0){
 		fetion_user_download_portrait(fxmain->user , fxmain->user->sipuri);
 		sprintf(filepath , "%s/%s.jpg" , config->iconPath , fxmain->user->sId);
-		pb = gdk_pixbuf_new_from_file_at_size(filepath , 50 , 50 , NULL);
+		pb = gdk_pixbuf_new_from_file_at_size(filepath,
+				USER_PORTRAIT_SIZE , USER_PORTRAIT_SIZE , NULL);
+
 		if(pb == NULL)
-			pb = gdk_pixbuf_new_from_file_at_size(filepath , 50 , 50 , NULL);
+			pb = gdk_pixbuf_new_from_file_at_size(filepath,
+					USER_PORTRAIT_SIZE, USER_PORTRAIT_SIZE, NULL);
+
+		gdk_threads_enter();
 		gtk_image_set_from_pixbuf(GTK_IMAGE(fxhead->portrait) , pb);
 		g_object_unref(pb);
 		gdk_threads_leave();
-	}
-	else
-	{
+	}else{
 		gdk_threads_enter();
 		dialog = gtk_message_dialog_new(GTK_WINDOW(fxmain->window),
 										GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -543,11 +570,14 @@ void* fx_head_change_portrait_thread(void* data)
 		gdk_threads_leave();
 
 		gdk_threads_enter();
-		bzero(filepath , sizeof(filepath));
 		sprintf(filepath , "%s/%s.jpg" , config->iconPath , fxmain->user->sId);
-		pb = gdk_pixbuf_new_from_file_at_size(filepath , 50 , 50 , NULL);
+		pb = gdk_pixbuf_new_from_file_at_size(filepath ,
+				USER_PORTRAIT_SIZE , USER_PORTRAIT_SIZE , NULL);
+
 		if(pb == NULL)
-			pb = gdk_pixbuf_new_from_file_at_size(filepath , 50 , 50 , NULL);
+			pb = gdk_pixbuf_new_from_file_at_size(filepath,
+					USER_PORTRAIT_SIZE , USER_PORTRAIT_SIZE , NULL);
+
 		gtk_image_set_from_pixbuf(GTK_IMAGE(fxhead->portrait) , pb);
 		g_object_unref(pb);
 		gdk_threads_leave();
